// CommPort.java

package pb.comm;


import javax.comm.*;
import javax.swing.*;
import java.io.*;
import java.util.TooManyListenersException;

public class CommPort implements SerialPortEventListener,
	CommPortOwnershipListener {
	
	static final int PORT_IDLE = 0;
	static final int PORT_XMITTING = 1;
	static final int PORT_ACK_WAIT = 1;
	int status = PORT_IDLE;

	private OutputStream os;
	private InputStream is;

	private CommPortIdentifier portId;
	private SerialPort sPort;

	private boolean open;

    LineReceiver lr = null;

	String response = "", buffer = "";
	Timer timeoutTimer;

	class TimerListener implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			status = PORT_IDLE;
		}
	}

    public CommPort() {
		open = false;
		timeoutTimer = new Timer(2000, new TimerListener());
		timeoutTimer.setRepeats(false);
	}

   /**
   Attempts to open a serial connection and streams using the parameters
   in the SerialParameters object. If it is unsuccessful at any step it
   returns the port to a closed state, throws a
   <code>SerialConnectionException</code>, and returns.

   Gives a timeout of 30 seconds on the portOpen to allow other applications
   to reliquish the port if have it open and no longer need it.
   */
	public void openConnection() throws SerialConnectionException {
		try {
		    portId = CommPortIdentifier.getPortIdentifier("COM1");
		}
		catch (NoSuchPortException e) {
	    	throw new SerialConnectionException(e.getMessage());
		}

		try {
		    sPort = (SerialPort)portId.open("SerialDemo", 30000);
		}
		catch (PortInUseException e) {
		    throw new SerialConnectionException(e.getMessage());
		}

		try {
		    sPort.setSerialPortParams(19200,SerialPort.DATABITS_8,
		    	SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);
		}
		catch (UnsupportedCommOperationException e) {
	    	throw new SerialConnectionException("Unsupported parameter");
		}

		// Open the input and output streams for the connection. If they won't
		// open, close the port before throwing an exception.
		try {
	    	os = sPort.getOutputStream();
		    is = sPort.getInputStream();
		}
		catch (IOException e) {
	    	sPort.close();
		    throw new SerialConnectionException("Error opening i/o streams");
		}

		// Add this object as an event listener for the serial port.
		try {
	    	sPort.addEventListener(this);
		}
		catch (TooManyListenersException e) {
		    sPort.close();
	    	throw new SerialConnectionException("too many listeners added");
		}

		// Set notifyOnDataAvailable to true to allow event driven input.
		sPort.notifyOnDataAvailable(true);

		// Set notifyOnBreakInterrup to allow event driven break handling.
		sPort.notifyOnBreakInterrupt(true);

		// Set receive timeout to allow breaking out of polling loop during
		// input handling.
		try {
		    sPort.enableReceiveTimeout(30);
		}
		catch (UnsupportedCommOperationException e) {
		}

		// Add ownership listener to allow ownership event handling.
		portId.addPortOwnershipListener(this);

		open = true;
    }

    public void serialEvent(SerialPortEvent e) {
	 	// Create a StringBuffer and int to receive input data.
		StringBuffer inputBuffer = new StringBuffer();
		int newData = 0;

		// Determine type of event.
		switch (e.getEventType()) {

		    // Read data until -1 is returned. If \r is received substitute
		    // \n for correct newline handling.
		    case SerialPortEvent.DATA_AVAILABLE:
			    while (newData != -1) {
			    	try {
			    	    newData = is.read();
					    if (newData == -1) {
							break;
					    }
                        if (lr != null) {
                        	lr.lineReceive("" + (char)newData);
                        }
			    	}
			    	catch (IOException ex) {
			    	    System.err.println(ex);
			    	    return;
		    	  	}
	   		    }
				break;
		}

    }

    public void ownershipChange(int type) {
		if (type == CommPortOwnershipListener.PORT_OWNERSHIP_REQUESTED) {
		}
    }

    public void sendString(String s) {
    	byte buf[] = new byte[s.length()];
    	s.getBytes(0,s.length(),buf,0);
    	status = PORT_XMITTING;
    	try {
	    	os.write(buf,0,buf.length);
	    	timeoutTimer.restart();
    	}
    	catch (IOException e) {
    		System.out.println("Error:"+e);
    	}
    }

    public boolean isIdle() {
    	return (status == PORT_IDLE);
    }

    public void registerReceiver(LineReceiver l) {
    	lr = l;
    }
}

