package pb.ir.analysis;


import pb.ir.irdata.Manuf;
import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.SignalParams;


public abstract class SignalAnalysis {
    protected double percentageError = 15;
    protected double fixedError = 150;

	protected Manuf manuf = null;

    protected IRBitstream bits;
    protected boolean generated = false;
    protected boolean valid = false;
    protected long signalCode = 0;
    protected String error = "";

    public SignalAnalysis() {
    }

    abstract public void generateSignal(long code, SignalParams params);

    abstract public boolean analyzeSignal(SignalParams params, IRBitstream bits);

    public void setManuf(Manuf m) {
		manuf = m;
	}

    public IRBitstream getIRBitstream() {
    	return bits;
    }
    public String getError() {
        return error;
    }

	public boolean isValid() {
		return valid;
	}

	public boolean isGenerated() {
		return generated;
	}

	public long getCode() {
		return signalCode;
	}

    public boolean closeEnough(int sigTime, int actTime) {
        double diff = Math.abs(1.0 * sigTime - actTime);
        if (diff / sigTime < percentageError / 100 || diff < fixedError)
            return true;

        return false;
    }
}
