package pb.ir.analysis;


import pb.ir.irdata.Manuf;
import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.SignalParams;


public class SignalAnalysisPulse1 extends SignalAnalysis {
    public SignalAnalysisPulse1() {
    	super();
    }

    public boolean analyzeSignal(SignalParams params, IRBitstream sig) {
    	bits = sig;

    	int maxOneP = 0, maxZeroP = 0;
        int minOneP = 2000, minZeroP = 2000;
        int maxSpace = 0, minSpace = 2000;

    	valid = false;

        int sigInx = 0;

        int expectedLen = params.codeLen * 2
            + 2 // Header
            + (params.stopP > 0 ? 2 : 0)    // Stop pulse, if any
            + (params.endP > 0 ? 1 : 0);   // End pulse if any
        if (bits.getBitstreamSize() < expectedLen) {
            error = "Expected " + expectedLen + " bits, got "
            	+ bits.getBitstreamSize();
            return false;
        }

        if (!closeEnough(params.headP, bits.getBitDuration(sigInx))
            || !closeEnough(params.headS, bits.getBitDuration(sigInx + 1))) {

            error = "Header pulse not detected - expected " + params.headP
                + "/" + params.headS + ", got " + bits.getBitDuration(sigInx) + "/"
                + bits.getBitDuration(sigInx + 1);
            return false;
        }

        bits.addBitType(IRBitstream.SIG_HDR, bits.getBitDuration(sigInx)
            + bits.getBitDuration(sigInx + 1));
        sigInx += 2;

        signalCode = 0;

        for (int i = 0; i < params.codeLen - 1; i++) {
        	if (closeEnough(params.oneP, bits.getBitDuration(sigInx))
            	&& closeEnough(params.oneS, bits.getBitDuration(sigInx + 1))) {

                if (bits.getBitDuration(sigInx) > maxOneP)
					maxOneP = bits.getBitDuration(sigInx);
                if (bits.getBitDuration(sigInx) < minOneP)
					minOneP = bits.getBitDuration(sigInx);
                if (bits.getBitDuration(sigInx + 1) > maxSpace)
					maxSpace = bits.getBitDuration(sigInx + 1);
                if (bits.getBitDuration(sigInx + 1) < minSpace)
					minSpace = bits.getBitDuration(sigInx + 1);

		        bits.addBitType(IRBitstream.SIG_ONE, bits.getBitDuration(sigInx)
        		    + bits.getBitDuration(sigInx + 1));
		        sigInx += 2;

                signalCode *= 2;
                signalCode++;
            }
        	else if (closeEnough(params.zeroP, bits.getBitDuration(sigInx))
            	&& closeEnough(params.zeroS, bits.getBitDuration(sigInx + 1))) {

                if (bits.getBitDuration(sigInx) > maxZeroP)
					maxZeroP = bits.getBitDuration(sigInx);
                if (bits.getBitDuration(sigInx) < minZeroP)
					minZeroP = bits.getBitDuration(sigInx);
                if (bits.getBitDuration(sigInx + 1) > maxSpace)
					maxSpace = bits.getBitDuration(sigInx + 1);
                if (bits.getBitDuration(sigInx + 1) < minSpace)
					minSpace = bits.getBitDuration(sigInx + 1);

		        bits.addBitType(IRBitstream.SIG_ZERO, bits.getBitDuration(sigInx)
        		    + bits.getBitDuration(sigInx + 1));
		        sigInx += 2;

                signalCode *= 2;
            }
            else {
            	error = "Bit " + i + " of " + bits.getBitDuration(sigInx) + "/"
                   	+ bits.getBitDuration(sigInx + 1)
   	            	+ " does not match One Signal(" + params.oneP + "/"
       	            + params.oneS + ") or Zero Signal(" + params.zeroP
           	        + "/" + params.zeroS + ")";
                return false;
            }
        }

        int i = params.codeLen - 1;
       	if (closeEnough(params.oneP, bits.getBitDuration(sigInx))
           	&& closeEnough(params.gap, bits.getBitDuration(sigInx + 1))) {

            if (bits.getBitDuration(sigInx) > maxOneP)
				maxOneP = bits.getBitDuration(sigInx);
            if (bits.getBitDuration(sigInx) < minOneP)
				minOneP = bits.getBitDuration(sigInx);

	        bits.addBitType(IRBitstream.SIG_ONE, bits.getBitDuration(sigInx)
       		    + params.oneS);
	        bits.addBitType(IRBitstream.SIG_GAP, bits.getBitDuration(sigInx + 1)
       		    - params.oneS);
            sigInx += 2;

            signalCode *= 2;
            signalCode++;
        }
       	else if (closeEnough(params.zeroP, bits.getBitDuration(sigInx))
           	&& closeEnough(params.gap, bits.getBitDuration(sigInx + 1))) {

            if (bits.getBitDuration(sigInx) > maxZeroP)
				maxZeroP = bits.getBitDuration(sigInx);
            if (bits.getBitDuration(sigInx) < minZeroP)
				minZeroP = bits.getBitDuration(sigInx);

	        bits.addBitType(IRBitstream.SIG_ZERO, bits.getBitDuration(sigInx)
       		    + params.zeroS);
	        bits.addBitType(IRBitstream.SIG_GAP, bits.getBitDuration(sigInx + 1)
       		    - params.zeroS);
            sigInx += 2;

            signalCode *= 2;
        }
        else {
           	error = "Bit " + i + " of " + bits.getBitDuration(sigInx) + "/"
               	+ bits.getBitDuration(sigInx + 1)
            	+ " does not match One Signal(" + params.oneP + "/"
   	            + params.gap + ") or Zero Signal(" + params.zeroP
       	        + "/" + params.gap + ")";
            return false;
        }

        bits.setSignalSetEnd(sigInx - 1);

        error = "So far, so good";
        valid = true;

        System.out.println("Bit timing stats: One=" + minOneP + "/" + maxOneP
        	+ ", Zero=" + minZeroP + "/" + maxZeroP + ", Space="
            + minSpace + "/" + maxSpace);
        return true;
    }

    public void generateSignal(long code, SignalParams params) {
    	bits = new IRBitstream();

        bits.addBitstream(params.headP);
        bits.addBitstream(params.headS);

        for (int i = 0; i < params.codeLen; i++) {
           	if ((code & (1L << (params.codeLen - i - 1))) == 0) {
            	if (i == params.codeLen - 1) {
			        bits.addBitstream(params.zeroP);
    		    	bits.addBitstream(params.gap);
                }
                else {
			        bits.addBitstream(params.zeroP);
    		    	bits.addBitstream(params.zeroS);
                }
            }
            else {
            	if (i == params.codeLen - 1) {
			        bits.addBitstream(params.oneP);
    		    	bits.addBitstream(params.gap);
                }
                else {
			        bits.addBitstream(params.oneP);
    		    	bits.addBitstream(params.oneS);
                }
            }
        }
        generated = true;
    }
}

