package pb.ir.analysis;


import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.SignalParams;


public class SignalAnalysisSpace1 extends SignalAnalysis {
	public SignalAnalysisSpace1() {
		super();
	}

	public boolean analyzeSignal(SignalParams params, IRBitstream sig) {
		valid = false;
		bits = sig;

		int sigInx = 0;

		int expectedLen = params.codeLen * 2
			+ 2 // Header
			+ (params.stopP > 0 ? 2 : 0)	// Stop pulse, if any
			+ (params.endP > 0 ? 1 : 0);   // End pulse if any
		if (bits.getBitstreamSize() < expectedLen) {
			error = "Expected " + expectedLen + " bits, got "
				+ bits.getBitstreamSize();
			return false;
		}

		if (!closeEnough(params.headP, bits.getBitDuration(sigInx))
			|| !closeEnough(params.headS, bits.getBitDuration(sigInx + 1))) {

			error = "Header pulse not detected - expected " + params.headP
				+ "/" + params.headS + ", got " + bits.getBitDuration(sigInx) + "/"
				+ bits.getBitDuration(sigInx + 1);
			return false;
		}
		bits.addBitType(IRBitstream.SIG_HDR, bits.getBitDuration(sigInx)
			+ bits.getBitDuration(sigInx + 1));
		sigInx += 2;

		signalCode = 0;

		for (int i = 0; i < params.codeLen; i++) {
			if (closeEnough(params.oneP, bits.getBitDuration(sigInx))
				&& closeEnough(params.oneS, bits.getBitDuration(sigInx + 1))) {

				bits.addBitType(IRBitstream.SIG_ONE, bits.getBitDuration(sigInx)
					+ bits.getBitDuration(sigInx + 1));
				sigInx += 2;
				signalCode *= 2;
				signalCode++;
			}
			else if (closeEnough(params.zeroP, bits.getBitDuration(sigInx))
				&& closeEnough(params.zeroS, bits.getBitDuration(sigInx + 1))) {

				bits.addBitType(IRBitstream.SIG_ZERO, bits.getBitDuration(sigInx)
					+ bits.getBitDuration(sigInx + 1));
				sigInx += 2;
				signalCode *= 2;
			}
			else {
				error = "Signal of " + bits.getBitDuration(sigInx) + "/" + bits.getBitDuration(sigInx + 1)
					+ " does not match One Signal(" + params.oneP + "/"
					+ params.oneS + ") or Zero Signal(" + params.zeroP
					+ "/" + params.zeroS + ")";
				return false;
			}
		}

		if (params.stopP > 0) {
			if (closeEnough(params.stopP, bits.getBitDuration(sigInx))
				&& closeEnough(params.stopS, bits.getBitDuration(sigInx + 1))) {

				bits.addBitType(IRBitstream.SIG_FTR, bits.getBitDuration(sigInx)
					+ bits.getBitDuration(sigInx + 1));

				sigInx += 2;
			}
			else {
				error = "Signal of " + bits.getBitDuration(sigInx) + "/"
					+ bits.getBitDuration(sigInx + 1)
					+ " does not match Stop Signal(" + params.stopP + "/"
					+ params.stopS + ")";
				return false;
			}
		}

		if (params.endP > 0) {
			if (closeEnough(params.endP, bits.getBitDuration(sigInx))) {
				bits.addBitType(IRBitstream.SIG_FTR,
					bits.getBitDuration(sigInx));

				sigInx++;
			}
		}

		bits.setSignalSetEnd(sigInx);

		// Duration of the gap
		bits.addBitType(IRBitstream.SIG_GAP, bits.getBitDuration(sigInx));
		int start = sigInx++;

		for (; sigInx < bits.getBitstreamSize(); sigInx++) {
			if (bits.getBitDuration(sigInx) > params.gap / 2) {
				bits.setSignalSet(start, sigInx);
				start = sigInx;
			}
		}
        if (start < sigInx - 1) {
			bits.setSignalSet(start, sigInx);
        }

		error = "So far, so good";
		valid = true;
		return true;
	}

	public void generateSignal(long code, SignalParams params) {
		bits = new IRBitstream();

		bits.addBitstream(params.headP);
		bits.addBitstream(params.headS);

		for (int i = 0; i < params.codeLen; i++) {
			if ((code & (1L << (params.codeLen - i - 1))) == 0) {
				bits.addBitstream(params.zeroP);
   				bits.addBitstream(params.zeroS);
			}
			else {
				bits.addBitstream(params.oneP);
   				bits.addBitstream(params.oneS);
			}
		}

		if (params.stopP != 0) {
			bits.addBitstream(params.stopP);
			bits.addBitstream(params.stopS);
		}

		if (params.endP != 0) {
			bits.addBitstream(params.endP);
		}

		bits.addBitstream(params.gap);

		generated = true;
	}
}

