package pb.ir.analysis;


import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.SignalParams;


public class SignalAnalysisSpace2 extends SignalAnalysis {
	public SignalAnalysisSpace2() {
		super();
	}

	public boolean analyzeSignal(SignalParams params, IRBitstream sig) {
		valid = false;
		bits = sig;

		int sigInx = 0;
		int runningTotal = 0;

		int expectedLen = params.codeLen * 2
			+ 2;   // Header
		if (bits.getBitstreamSize() < expectedLen) {
			error = "Expected " + expectedLen + " bits, got "
				+ bits.getBitstreamSize();
			return false;
		}

		if (!closeEnough(params.headP, bits.getBitDuration(sigInx))
			|| !closeEnough(params.headS, bits.getBitDuration(sigInx + 1))) {

			error = "Header pulse not detected - expected " + params.headP
				+ "/" + params.headS + ", got " + bits.getBitDuration(sigInx) + "/"
				+ bits.getBitDuration(sigInx + 1);
			return false;
		}
		bits.addBitType(IRBitstream.SIG_HDR, bits.getBitDuration(sigInx)
			+ bits.getBitDuration(sigInx + 1));
		sigInx += 2;

		signalCode = 0;

		for (int i = 0; i < params.codeLen; i++) {
			if (closeEnough(params.oneP, bits.getBitDuration(sigInx))
				&& closeEnough(params.oneS, bits.getBitDuration(sigInx + 1))) {

				bits.addBitType(IRBitstream.SIG_ONE, bits.getBitDuration(sigInx)
					+ bits.getBitDuration(sigInx + 1));
				runningTotal += bits.getBitDuration(sigInx) + bits.getBitDuration(sigInx + 1);
				sigInx += 2;
				signalCode *= 2;
				signalCode++;
			}
			else if (closeEnough(params.zeroP, bits.getBitDuration(sigInx))
				&& closeEnough(params.zeroS, bits.getBitDuration(sigInx + 1))) {
				bits.addBitType(IRBitstream.SIG_ZERO, bits.getBitDuration(sigInx)
					+ bits.getBitDuration(sigInx + 1));
				runningTotal += bits.getBitDuration(sigInx) + bits.getBitDuration(sigInx + 1);
				sigInx += 2;
				signalCode *= 2;
			}
			else {
				error = "Signal of " + bits.getBitDuration(sigInx) + "/" + bits.getBitDuration(sigInx + 1)
					+ " does not match One Signal(" + params.oneP + "/"
					+ params.oneS + ") or Zero Signal(" + params.zeroP
					+ "/" + params.zeroS + ")";
				return false;
			}
		}

	   	if (closeEnough(params.oneP, bits.getBitDuration(sigInx))
		   	&& bits.getBitDuration(sigInx + 1) + runningTotal > params.gap / 2) {
			bits.addBitType(IRBitstream.SIG_FTR, bits.getBitDuration(sigInx));
			bits.addBitType(IRBitstream.SIG_GAP, bits.getBitDuration(sigInx + 1));
			bits.setSignalSetEnd(sigInx + 1);
			sigInx += 2;
		}
		else {
		   	error = "Signal of " + bits.getBitDuration(sigInx) + "/"
				+ (bits.getBitDuration(sigInx + 1) + runningTotal)
			   	+ " does not match End Signal(" + params.oneP + "/"
				+ params.gap + ")";
			return false;
		}

		int start = sigInx - 1;

		if (params.stopP > 0) {
		   	if (closeEnough(params.stopP, bits.getBitDuration(sigInx))
			   	&& closeEnough(params.stopS, bits.getBitDuration(sigInx + 1))) {

				bits.addBitType(IRBitstream.SIG_FTR,
					bits.getBitDuration(sigInx)
                    + bits.getBitDuration(sigInx + 1));
                sigInx += 2;
			}
		}

		if (closeEnough(params.endP, bits.getBitDuration(sigInx))) {
			bits.addBitType(IRBitstream.SIG_FTR, bits.getBitDuration(sigInx));
            sigInx++;
		}

        if (start < sigInx - 1) {
			bits.setSignalSet(start, sigInx);
        }
		start = sigInx;
        sigInx++;

		for (; sigInx < bits.getBitstreamSize(); sigInx++) {
			if (bits.getBitDuration(sigInx) > params.gap / 2) {
				bits.setSignalSet(start, sigInx);
				start = sigInx;
			}
		}
        if (start < sigInx - 1) {
			bits.setSignalSet(start, sigInx);
        }

		error = "So far, so good";
		valid = true;
		return true;
	}

	public void generateSignal(long code, SignalParams params) {
		bits = new IRBitstream();
		int runningTotal = 0;

		bits.addBitstream(params.headP);
		bits.addBitstream(params.headS);
		runningTotal += params.headP + params.headS;

		for (int i = 0; i < params.codeLen; i++) {
		   	if ((code & (1L << (params.codeLen - i - 1))) == 0) {
				bits.addBitstream(params.zeroP);
   				bits.addBitstream(params.zeroS);
				runningTotal += params.zeroP + params.zeroS;
			}
			else {
				bits.addBitstream(params.oneP);
   				bits.addBitstream(params.oneS);
				runningTotal += params.oneP + params.oneS;
			}
		}

		if (params.endP != 0) {
			bits.addBitstream(params.endP);
			runningTotal += params.endP;

			bits.addBitstream(params.gap - runningTotal);
		}

		if (params.stopP != 0) {
			bits.addBitstream(params.stopP);
			bits.addBitstream(params.stopS);
			runningTotal = params.stopP + params.stopS;
		}

		if (params.endP != 0) {
			bits.addBitstream(params.endP);
			runningTotal += params.endP;
		}

		bits.addBitstream(params.gap - runningTotal);

		generated = true;
	}
}

