package pb.ir.irdata;


public class IRBitstream {
    public static final int SIG_NONE = 0;
    public static final int SIG_HDR = 1;
    public static final int SIG_ONE = 2;
    public static final int SIG_ZERO = 3;
    public static final int SIG_FTR = 4;
    public static final int SIG_GAP = 5;

    static int chunkSize = 100;

    protected int[] bitstreamTiming = new int[chunkSize];
    protected int bitstreamSize = 0;
    protected int bitstreamSpan = 0;

    protected int[] bitType = new int[chunkSize];
    protected int[] bitTypeDuration = new int[chunkSize];
    protected int bitTypeSize = 0;

    protected int[] signalSetFrom = new int[chunkSize];
    protected int[] signalSetTo = new int[chunkSize];
	protected int signalSetSize = 0;

    public void addBitstream(int duration) {
    	if (bitstreamSize + 1 > bitstreamTiming.length) {
        	int[] temp = bitstreamTiming;
            bitstreamTiming = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, bitstreamTiming, 0, temp.length);
        }

        bitstreamTiming[bitstreamSize++] = duration;
        bitstreamSpan += duration;
    }

    public int getBitstreamSpan() {
    	return bitstreamSpan;
    }

    public int getBitstreamSize() {
    	return bitstreamSize;
    }

    public int getBitDuration(int inx) {
    	return bitstreamTiming[inx];
    }

    public int getBitTypeSize() {
    	return bitTypeSize;
    }

    public int getBitType(int inx) {
    	return bitType[inx];
    }

    public int getBitTypeDuration(int inx) {
    	return bitTypeDuration[inx];
    }

    public int getSignalSetSize() {
    	return signalSetSize;
    }

    public int getSignalSetFrom(int inx) {
    	return signalSetFrom[inx];
    }

    public int getSignalSetTo(int inx) {
    	return signalSetTo[inx];
    }

    public void setSignalSet(int start, int end) {
    	if (signalSetSize + 1 > signalSetFrom.length) {
        	int[] temp = signalSetFrom;
            signalSetFrom = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, signalSetFrom, 0, temp.length);

        	temp = signalSetTo;
            signalSetTo = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, signalSetTo, 0, temp.length);
        }

        signalSetFrom[signalSetSize] = start;
        signalSetTo[signalSetSize] = end;
        signalSetSize++;
    }

    public void setSignalSetEnd(int inx) {
    	if (signalSetSize + 1 > signalSetFrom.length) {
        	int[] temp = signalSetFrom;
            signalSetFrom = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, signalSetFrom, 0, temp.length);

        	temp = signalSetTo;
            signalSetTo = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, signalSetTo, 0, temp.length);
        }

        if (signalSetSize == 0) {
        	signalSetFrom[signalSetSize] = 0;
        }
        else {
        	signalSetFrom[signalSetSize] = signalSetTo[signalSetSize - 1] + 1;
            if (signalSetFrom[signalSetSize] % 2 == 0) {
            	signalSetFrom[signalSetSize]++;
            }
        }
        signalSetTo[signalSetSize] = inx;
        signalSetSize++;
    }

    public void addBitType(int type, int duration) {
    	if (bitTypeSize + 1 > bitType.length) {
        	int[] temp = bitType;
            bitType = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, bitType, 0, temp.length);

        	temp = bitTypeDuration;
            bitTypeDuration = new int[temp.length + chunkSize];
            System.arraycopy(temp, 0, bitTypeDuration, 0, temp.length);
        }

        bitType[bitTypeSize] = type;
        bitTypeDuration[bitTypeSize++] = duration;
    }
}
