package pb.ir.irdata;


import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import pb.xml.XMLEncode;
import pb.xml.parser.Handler;
import pb.xml.parser.Parser;
import pb.xml.parser.tiny.TinyParser;
import pb.util.OrdList;


public class IRData extends Object {
	OrdList manufList = new OrdList();
	Manuf currentManuf;
	IRDevice currentDevice;
	IRFunction currentFunction;

	class IRHandler extends Handler {
		IRHandler() {
			super("");
		}

		public void processStartTag(String path, String name,
			Hashtable attrs) {
			if (path.equals("IRDoc/Manuf")) {
				String mName = (String)(attrs.get("name"));
				currentManuf = new Manuf();
                currentManuf.name = mName;
				manufList.put(mName, currentManuf);
			}
			else if (path.equals("IRDoc/Manuf/Signal")) {
				String param;
				String value;
				param = "freq";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.freq = val;
				}
				param = "type";
				if ((value = (String)(attrs.get(param))) != null) {
                	currentManuf.signal.type = value;
				}
				param = "length";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.codeLen = val;
				}
				param = "headP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.headP = val;
				}
				param = "headS";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.headS = val;
				}
				param = "oneP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.oneP = val;
				}
				param = "oneS";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.oneS = val;
				}
				param = "zeroP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.zeroP = val;
				}
				param = "zeroS";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.zeroS = val;
				}
				param = "endP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.endP = val;
				}
				param = "stopP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.stopP = val;
				}
				param = "stopS";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.stopS = val;
				}
				param = "endStopP";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.endStopP = val;
				}
				param = "gap";
				if ((value = (String)(attrs.get(param))) != null) {
					int val = Integer.parseInt(value);
					currentManuf.signal.gap = val;
				}
			}
			else if (path.equals("IRDoc/Manuf/Device")) {
				String dName = (String)(attrs.get("name"));
				currentDevice = new IRDevice(dName);
				currentManuf.addDevice(currentDevice);
			}
			else if (path.equals("IRDoc/Manuf/Device/Function")) {
				String fCode = (String)(attrs.get("code"));
				currentFunction = new IRFunction();
				currentFunction.code = Long.parseLong(fCode, 16);
			}
			else if (path.equals("IRDoc/Manuf/Module")) {
				String mName = (String)(attrs.get("name"));
				currentManuf.module = mName;
			}
			else if (path.equals("IRDoc")) {
			}
			else {
				System.out.println("S:" + path);
			}
		}

		public void processEndTag(String path, String name, String value) {
			if (path.equals("IRDoc/Manuf")) {
				if (currentManuf != null) {
					currentManuf.build();
					currentManuf = null;
				}
			}
			else if (path.equals("IRDoc/Manuf/Device")) {
				currentDevice = null;
			}
			else if (path.equals("IRDoc/Manuf/Device/Function")) {
				currentDevice.addFunction(currentFunction);
				currentFunction = null;
			}
		}

		public void setValue(String name, String value) {
			if (name.equals("/IRDoc/Manuf/Device/Function")) {
				currentFunction.name = value;
			}
		}
	}

	public IRData() {
	}

	public void clear() {
		manufList = new OrdList();
	}

	public void load(InputStream is) throws Exception {
    	clear();
        
		IRHandler h = new IRHandler();
		try {
			Parser mp = new TinyParser(is, h);
		}
		catch (Exception ex) {
			System.out.println("Error:" + ex);
			ex.printStackTrace();
			throw ex;
		}
	}

    public void save(OutputStream os) throws Exception {
    	PrintWriter pw = new PrintWriter(new OutputStreamWriter(os));

		pw.println("<?xml version=\"1.0\"?>");
		pw.println();
		pw.println("<IRDoc>");

		for (Enumeration mit = getManufList(); mit.hasMoreElements(); ) {
            Manuf m = (Manuf)(mit.nextElement());
			pw.println("  <Manuf name=\"" + m.getName() + "\">");

            if (m.module != null) {
				pw.println("    <Module name=\"" + m.module + "\"/>");
            }

            pw.println("    <Signal freq=\"" + m.signal.freq
            	+ "\" type=\"" + m.signal.type
                + "\" length=\"" + m.signal.codeLen
                + "\" gap=\"" + m.signal.gap + "\"");

            pw.println("      headP=\"" + m.signal.headP
            	+ "\" headS=\"" + m.signal.headS + "\"");
            pw.println("      oneP=\"" + m.signal.oneP
            	+ "\" oneS=\"" + m.signal.oneS + "\"");
            pw.print("      zeroP=\"" + m.signal.zeroP
            	+ "\" zeroS=\"" + m.signal.zeroS + "\"");

            if (m.signal.stopP > 0) {
            	pw.println();
	            pw.print("      stopP=\"" + m.signal.stopP
    	        	+ "\" stopS=\"" + m.signal.stopS + "\"");
            }

            if (m.signal.endP > 0) {
            	pw.println();
	            pw.print("      endP=\"" + m.signal.endP + "\"");
            }

            pw.println("/>");

            for (Enumeration dit = m.getDeviceList(); dit.hasMoreElements(); ) {
                IRDevice dev = (IRDevice)(dit.nextElement());
                pw.println("    <Device name=\""
                	+ XMLEncode.xmlEncode(dev.getName()) + "\">");

                for (Enumeration fit = dev.getFuncListx();
                	fit.hasMoreElements(); ) {

                	IRFunction func = (IRFunction)(fit.nextElement());

                    pw.println("      <Function code=\""
                    	+ Long.toHexString(func.code) + "\">"
                    	+ XMLEncode.xmlEncode(func.name) + "</Function>");
                }
				pw.println("    </Device>");
            }
			pw.println("  </Manuf>");
        }

		pw.println("</IRDoc>");
        pw.flush();
    }

	public Manuf getManuf(String manufName) {
		return (Manuf)(manufList.get(manufName));
	}

	public Enumeration getManufList() {
		return manufList.values();
	}

	public String getModule(String manufName) {
		Manuf m = (Manuf)(manufList.get(manufName));
		if (m == null)
			return null;
		return m.module;
	}

	public SignalParams getSignal(String manufName) {
		Manuf m = (Manuf)(manufList.get(manufName));
		if (m == null)
			return null;
		return m.signal;
	}

	public IRFunction getFunc(String manufName, String deviceName,
		String funcName) {

		Manuf m = (Manuf)(manufList.get(manufName));
		if (m == null)
			return null;

		return m.getFunc(deviceName, funcName);
	}

	public Enumeration getFuncList(String manufName, String deviceName) {
		Manuf m = (Manuf)(manufList.get(manufName));
		if (m == null)
			return null;

		return m.getFuncList(deviceName);
	}

	public Enumeration getDeviceList(String manufName) {
		Manuf m = (Manuf)(manufList.get(manufName));
		if (m == null)
			return null;

		return m.getDeviceList();
	}
}
