package pb.ir.ui;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JPanel;

import pb.ir.analysis.SignalAnalysis;
import pb.ir.irdata.IRData;
import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.IRDevice;
import pb.ir.irdata.IRFunction;
import pb.ir.irdata.Manuf;
import pb.ir.irdata.SignalParams;


public class IRMainUI extends JPanel {
	JPanel tracePanel, paramPanel;
    IRData irData = new IRData();
    JComboBox cbManuf, cbDev, cbFunc;
    JTextField tfFreq;
    JTextField tfCodeLen, tfCode, tfGap, tfHeadP, tfHeadS;
    JTextField tfOneP, tfOneS, tfZeroP, tfZeroS, tfEndP, tfStopP, tfStopS;
    JComboBox cbSigType;

    TreeMap manufModule = new TreeMap();

    IRBitstream bitstream = new IRBitstream();

    boolean sneakyChange = false;
    
    // Constants for the trace panel
    static final int traceLMargin = 40;
    static final int traceRMargin = 30;
    static final int maxBreak = 10000;
    static final int maxTraces = 5;
    static final int fullY1 = 15, fullY2 = 35;
    static final int fullTypeY1 = 40, fullTypeY2 = 50;


    static final double picScale = 9.4;

    static final int debugLevel = 0;

    class MyCanvas extends JPanel {
    	boolean hairline = true;
        int hairline1X = traceLMargin, hairline2X = traceLMargin + 10;
        boolean inited = false;
        int grabbed = 0;
        double timeWidth = 2000;
        int fullTimeWidth = 20000;
        double scale = 1;

    	public void paint(Graphics g) {

        	if (debugLevel > 0) {
            	System.out.println("Trace Panel: paint called");
            }

        	if (!inited) {
            	// Add listeners to handle moving hairlines
            	addMouseListener( new MouseListener() {
					public void mousePressed(MouseEvent e) {
                       	if (e.getX() > hairline1X - 2
                        	&& e.getX() < hairline1X + 2) {

                           	grabbed = 1;
                        }
                       	else if (e.getX() > hairline2X - 2
                        	&& e.getX() < hairline2X + 2) {

                           	grabbed = 2;
                        }
                    }

				    public void mouseReleased(MouseEvent e) {
                       	grabbed = 0;
                    }

				    public void mouseEntered(MouseEvent e) {}

				    public void mouseExited(MouseEvent e) {}

				    public void mouseClicked(MouseEvent e) {}
               	});

            	addMouseMotionListener( new MouseMotionListener() {
					public void mouseMoved(MouseEvent e) {}

				    public void mouseDragged(MouseEvent e) {
                       	if (grabbed != 0) {
                           	if (grabbed == 1) {
                            	int oldX = hairline1X;
                                hairline1X = e.getX();
                                if (hairline1X < traceLMargin)
                                   	hairline1X = traceLMargin;
                                if (hairline1X > getBounds().width - traceRMargin)
                                   	hairline1X = getBounds().width - traceRMargin;
                                repaint(Math.min(oldX, hairline1X), 0,
                                   	Math.abs(oldX - hairline1X),
                                    getBounds().height);
                                repaint(0, 0, getBounds().width, 20);
                                repaint(0, getBounds().height - 20,
                                   	getBounds().width, 20);
                            }
                            else if (grabbed == 2) {
                             	int oldX = hairline2X;
                                hairline2X = e.getX();
                                if (hairline2X < traceLMargin)
                                   	hairline2X = traceLMargin;
                                if (hairline2X > getBounds().width - traceRMargin)
                                  	hairline2X = getBounds().width - traceRMargin;
                                repaint(Math.min(oldX, hairline2X), 0,
                                   	Math.abs(oldX - hairline2X),
                                    getBounds().height);
                                repaint(0, 0, getBounds().width, 20);
                                repaint(0, getBounds().height - 20,
                                   	getBounds().width, 20);
                            }
                        }
                    }
              	});

            	inited = true;
            }

        	super.paint(g);
			g.setColor(Color.yellow);

            Font f = new Font("Helvetica", Font.BOLD, 10);
            FontMetrics fm = g.getFontMetrics(f);

            int w = getBounds().width;

            boolean is1 = true;

            int x, prevX = traceLMargin, ttl = 0;

            // -----------------------------------------------------------------
            // Draw trace for full signal
            //

        	if (debugLevel > 0) {
            	System.out.println("Trace Panel:   Drawing main trace");
            }

            // Ignore last count as it is usually a long period of silence.
            if (bitstream.getBitstreamSize() > 1) {
            	fullTimeWidth = bitstream.getBitstreamSpan()
            		- bitstream.getBitDuration(bitstream.getBitstreamSize() - 1);
            }

//System.out.println("ftw="+fullTimeWidth);

        	if (debugLevel > 0) {
            	System.out.println("Trace Panel:     Drawing trace line");
            }

            g.setColor(Color.white);
            g.drawLine(0, fullY2, traceLMargin, fullY2);

            prevX = traceLMargin;
            ttl = 0;
            is1 = true;

            // Draw entire stream.
            for (int inx = 0; inx < bitstream.getBitstreamSize() - 1; inx++) {
            	ttl += bitstream.getBitDuration(inx);

            	x = ((w - traceLMargin - traceRMargin) * ttl)
                	/ fullTimeWidth + traceLMargin;

                g.drawLine(prevX, fullY1, prevX, fullY2);
                g.drawLine(prevX, is1 ? fullY1 : fullY2, x, is1 ? fullY1 : fullY2);
                prevX = x;
                is1 = !is1;
            }

            if (is1) {
	            g.drawLine(prevX, fullY1, w, fullY1);
            }
            else {
	            g.drawLine(prevX, fullY1, prevX, fullY2);
	            g.drawLine(prevX, fullY2, w, fullY2);
            }

        	if (debugLevel > 0) {
            	System.out.println("Trace Panel:     Drawing bit type bands");
            }

            // Draw bit types for first set in full trace
            ttl = 0;
            for (int inx = 0; inx < bitstream.getBitTypeSize(); inx++) {
            	int x1 = ((w - traceLMargin - traceRMargin) * ttl)
                	/ fullTimeWidth
                	+ traceLMargin;
            	ttl += bitstream.getBitTypeDuration(inx);
            	int x2 = ((w - traceLMargin - traceRMargin) * ttl)
                	/ fullTimeWidth + traceLMargin;

                Color c = Color.white;
          	    switch (bitstream.getBitType(inx)) {
	   	        	case IRBitstream.SIG_HDR :
    	            	c = new Color(64, 64, 128);
        	            break;
    	   			case IRBitstream.SIG_ONE :
	           			c = new Color(10, 128, 10);
               			break;
		       		case IRBitstream.SIG_ZERO :
    	      	    	c = new Color(128, 20, 20);
          	            break;
        	    	case IRBitstream.SIG_FTR :
          		    	c = new Color(10, 96, 128);
               		    break;
    	        	case IRBitstream.SIG_GAP :
        	           	c = new Color(64, 64, 64);
        		        break;
			    }
                g.setColor(c);
                g.fillRect(x1, fullTypeY1, x2 - x1 - 1,
                	fullTypeY2 - fullTypeY1);
            }

            // -----------------------------------------------------------------
            // Draw individual traces
            //
        	if (debugLevel > 0) {
            	System.out.println("Trace Panel:     Drawing individual traces");
            }

            g.setColor(Color.yellow);

            int maxSamples = Math.min(bitstream.getSignalSetSize(), maxTraces);

            int pitch = 1;

            if (maxSamples > 0 ) {
            	pitch = (getBounds().height - 50 - fullY1)
            	/ (4 * maxSamples);
            }

            int total = 0;
            int elapsed = 0;
            int bitInx = 0;
	        for (int sInx = 0; sInx < maxSamples; sInx++) {
	            prevX = traceLMargin;
	            is1 = true;

                if (sInx == 0) {
	    	    	for (int i = bitstream.getSignalSetFrom(sInx)
                    	+ (sInx > 0 ? 1 : 0);
    	    	    	i < bitstream.getSignalSetTo(sInx); i++) {

                        total += bitstream.getBitDuration(i);
        	        }
		            timeWidth = total;
                }

            	int y = pitch * ((4 * sInx) + 1) + 50;
	            g.drawLine(0, y + pitch, traceLMargin, y + pitch);
                ttl = 0;
                int bitTtl = 0;
                x = 0;

                if (sInx > 0)
                	elapsed += bitstream.getBitDuration(
                    	bitstream.getSignalSetFrom(sInx));

//System.out.println("from " + (signal.sampleList[sInx] + (sInx > 0 ? 1 : 0)) + " to " + (signal.sampleList[sInx + 1]));
    	    	for (int i = bitstream.getSignalSetFrom(sInx) + (sInx > 0 ? 1 : 0);
        	    	i < bitstream.getSignalSetTo(sInx); i++) {
//System.out.println(i + ". sig="+signal.fullSignal[i]+",cum="+ttl);
	            	ttl += bitstream.getBitDuration(i);
    	        	x = (int)(Math.round(((w - traceLMargin - traceRMargin) * ttl) / timeWidth))
        	        	+ traceLMargin;

                    g.setColor(Color.yellow);
	                g.drawLine(prevX, y, prevX, y + pitch);
    	            g.drawLine(prevX, is1 ? y : y + pitch, x, is1 ? y : y + pitch);

    	            prevX = x;
        	        is1 = !is1;
                }

//System.out.println("span is " + elapsed + " to " + (elapsed+ttl));

   	            g.drawLine(prevX, y + pitch, x, y);
	            g.drawLine(prevX, is1 ? y : y + pitch, w, is1 ? y : y + pitch);

	        	if (debugLevel > 0) {
    	        	System.out.println(
                    	"Trace Panel:       Drawing individual bit types");
        	    }

                bitInx = 0;
                int bitElapsed = 0;
                bitTtl = 0;

                for (; bitInx < bitstream.getBitTypeSize(); bitInx++) {
                	if (bitstream.getBitTypeDuration(bitInx) + bitElapsed
                    	> elapsed) {

                    	break;
                    }

                    bitElapsed += bitstream.getBitTypeDuration(bitInx);
                }

                int digInx = 0;
                int digit = 0;
                int digX = traceLMargin;
                for (; bitInx < bitstream.getBitTypeSize(); bitInx++) {

                	if (bitElapsed + bitTtl
                    	> elapsed + ttl) {

                    	break;
                    }

                	int bX1 = (int)Math.round(((w - traceLMargin - traceRMargin)
   	                	* bitTtl) / timeWidth) + traceLMargin;

                    bitTtl += bitstream.getBitTypeDuration(bitInx);
                	int bX2 = (int)Math.round(((w - traceLMargin - traceRMargin)
   	                	* bitTtl) / timeWidth) + traceLMargin;
                    if (bX2 > w - traceRMargin) {
                    	bX2 = w - traceRMargin;
                    }

                    Color c = Color.white;
            	    switch (bitstream.getBitType(bitInx)) {
	    	        	case IRBitstream.SIG_HDR :
   			            	c = new Color(64, 64, 128);
                            digInx = 0;
                            digit = 0;
  	    		            break;
            			case IRBitstream.SIG_ONE :
                   			c = new Color(10, 128, 10);
                            if (digInx == 0)
                               	digX = bX1;
                            digit *= 2;
                            digit++;
                            digInx++;
                   			break;
                		case IRBitstream.SIG_ZERO :
	            	    	c = new Color(128, 20, 20);
                            if (digInx == 0)
                               	digX = bX1;
                            digit *= 2;
                            digInx++;
   		    	            break;
       			    	case IRBitstream.SIG_FTR :
                            digInx = 0;
                            digit = 0;
       	    		    	c = new Color(10, 96, 128);
   	            		    break;
	    	        	case IRBitstream.SIG_GAP :
                            digInx = 0;
                            digit = 0;
   			            	c = new Color(64, 64, 64);
   	    		            break;
	                }

                    if (digInx == 4) {
                    	g.setColor(Color.white);
                        g.drawLine(digX - 1, y + pitch * 3 / 2, digX - 1, y + pitch * 2);
                        g.drawLine(bX2 - 1, y + pitch * 3 / 2, bX2 - 1, y + pitch * 2);
                        String s = Integer.toHexString(digit).toUpperCase();
                        g.drawString(s,
                        	(bX2 + digX - fm.stringWidth(s)) / 2,
                            y + pitch * 2);
                        digit = 0;
                        digInx = 0;
                    }

                    g.setColor(c);

                    g.fillRect(bX1, y + pitch * 2, bX2 - bX1 - 1, pitch);

                }
                g.setColor(Color.yellow);

                elapsed += ttl;
                if (sInx > 0 && sInx < bitstream.getSignalSetSize())
                	elapsed += bitstream.getBitDuration(
                    	bitstream.getSignalSetFrom(sInx));
            }

        	if (debugLevel > 0) {
            	System.out.println("Trace Panel:     Drawing hairlines");
            }

            if (hairline) {
            	g.setColor(new Color(128, 128, 128));
                g.drawLine(hairline1X, 0, hairline1X, getBounds().height);
                int fullVal1 = (int)(Math.round(scale * (hairline1X - traceLMargin) * fullTimeWidth
                	/ (getBounds().width - traceLMargin - traceRMargin)));
                g.drawString("" + fullVal1, hairline1X + 2, 10);
                int val1 = (int)(Math.round(scale * (hairline1X - traceLMargin) * timeWidth
                	/ (getBounds().width - traceLMargin - traceRMargin)));
                g.drawString("" + val1, hairline1X + 2, getBounds().height - 2);

            	g.setColor(new Color(96, 96, 96));
                g.drawLine(hairline2X, 0, hairline2X, getBounds().height);
                int fullVal2 = (int)(Math.round(scale * (hairline2X - traceLMargin) * fullTimeWidth
                	/ (getBounds().width - traceLMargin - traceRMargin)));
                g.drawString("" + fullVal2, hairline2X + 2, 10);
                int val2 = (int)(Math.round(scale * (hairline2X - traceLMargin) * timeWidth
                	/ (getBounds().width - traceLMargin - traceRMargin)));
                g.drawString("" + val2, hairline2X + 2, getBounds().height - 2);

            	g.setColor(new Color(204, 204, 204));
                int fullDiff = fullVal2 - fullVal1;
                g.drawString("" + fullDiff, 2,
                	10);
                int diff = val2 - val1;
                g.drawString("" + diff, 2,
                	getBounds().height - 2);
            }
        }

        public Dimension getPreferredSize() {
        	return new Dimension(10, 10);
        }

        public Dimension getMinimumSize() {
        	return new Dimension(10, 10);
        }
    }

	class FormLayout implements LayoutManager {
    	int hGap = 5, vGap = 10;
        Dimension mySize = new Dimension();

		public FormLayout() {}

	    public void addLayoutComponent(String name, Component comp) {}

	    public void removeLayoutComponent(Component comp) {}

	    public Dimension preferredLayoutSize(Container parent) {
			return mySize;
	    }

	    public Dimension minimumLayoutSize(Container parent) {
			return mySize;
	    }

	    public void layoutContainer(Container parent) {
        	Component[] compList = parent.getComponents();

            int maxLblW = 0, maxFldW = 0;
            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();
                if (i % 2 == 0) {
                	// label
                    if (d.width > maxLblW)
                    	maxLblW = d.width;
                }
                else {
                	// field
                    if (d.width > maxFldW)
                    	maxFldW = d.width;
                }
            }

            int x = parent.getInsets().left;
            int y = parent.getInsets().top;
            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();
                if (i % 2 == 0) {
                	// label
                    Dimension d2 = d;
                    if (compList.length > i + 1)
                    	d2 = compList[i + 1].getPreferredSize();
					compList[i].setBounds(x + maxLblW - d.width,
                    	y + (d2.height - d.height) / 2,
                    	d.width, d.height);
                }
                else {
                	// field
					compList[i].setBounds(x + maxLblW + hGap, y,
                    	d.width, d.height);
                    y += d.height + vGap;
                    if (x + maxLblW + hGap + d.width + parent.getInsets().right
                    	> mySize.width)

                    	mySize.width = x + maxLblW + hGap + d.width
                        	+ parent.getInsets().right;
                }
            }
            mySize.height = y + parent.getInsets().bottom;
	    }
	}

	class ProportionLayout implements LayoutManager {
    	int hGap = 5, vGap = 10;
        Dimension mySize = new Dimension();

		public ProportionLayout() {}

	    public void addLayoutComponent(String name, Component comp) {}

	    public void removeLayoutComponent(Component comp) {}

	    public Dimension preferredLayoutSize(Container parent) {
			return mySize;
	    }

	    public Dimension minimumLayoutSize(Container parent) {
			return mySize;
	    }

	    public void layoutContainer(Container parent) {
        	Component[] compList = parent.getComponents();

            int totalW = 0, maxH = 0;
            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();
                totalW += d.width;
                if (maxH < d.height)
                	maxH = d.height;
            }

            Insets insets = parent.getInsets();
            Rectangle size = parent.getBounds();
            int x = insets.left;
            int y = insets.top;
            int h = size.height - insets.top - insets.bottom;
            int availableWidth = size.width - insets.left - insets.right
            	- hGap * (compList.length - 1);
            int w;

            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();

                w = d.width * availableWidth / totalW;
                compList[i].setBounds(x, y, w, h);
                x += w;
                x += hGap;
            }

            mySize.width = insets.left + insets.right
            	+ hGap * (compList.length - 1) + totalW;
            mySize.height = insets.top + insets.bottom + maxH;
	    }
	}

	class MyGridLayout implements LayoutManager {
    	int cols, hGap = 5, vGap = 10;
        Dimension mySize = new Dimension();

		public MyGridLayout(int cols, int hGap, int vGap) {
        	this.cols = cols;
            this.hGap = hGap;
            this.vGap = vGap;
        }

	    public void addLayoutComponent(String name, Component comp) {}

	    public void removeLayoutComponent(Component comp) {}

	    public Dimension preferredLayoutSize(Container parent) {
			return mySize;
	    }

	    public Dimension minimumLayoutSize(Container parent) {
			return mySize;
	    }

	    public void layoutContainer(Container parent) {
        	Component[] compList = parent.getComponents();

			int[] rowH = new int[(compList.length - 1) / cols + 1];
            int[] colW = new int[cols];

            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();
                int row = i / cols;
                int col = i % cols;
                if (d.height > rowH[row])
                	rowH[row] = d.height;
                if (d.width > colW[col])
                	colW[col] = d.width;
            }

            int x = parent.getInsets().left;
            int y = parent.getInsets().top;
            for (int i = 0; i < compList.length; i++) {
            	Dimension d = compList[i].getPreferredSize();
                int row = i / cols;
                int col = i % cols;

                if (col == 0) {
                	x = parent.getInsets().left;
                    if (row > 0)
	                    y += rowH[row - 1];
                }
                else {
                   	x += colW[col - 1];
                }

				compList[i].setBounds(x + colW[col] - d.width + hGap * col,
                   	y + (rowH[row] - d.height) / 2 + vGap * row,
                   	d.width, d.height);
            }

            mySize.width = hGap * (colW.length - 1) + parent.getInsets().left
            	+ parent.getInsets().right;
            for (int i = 0; i < colW.length; i++) {
            	mySize.width += colW[i];
            }
            mySize.height = vGap * (rowH.length - 1) + parent.getInsets().top
            	+ parent.getInsets().bottom;
            for (int i = 0; i < rowH.length; i++) {
            	mySize.height += rowH[i];
            }
	    }
	}

	public IRMainUI(Container frame) {
    	initUI(frame);
    }

    public void clear() {
		irData.clear();
		refreshDisplay();
    }

    public void load(InputStream is) throws Exception {
		irData.load(is);
		refreshDisplay();
    }

    private void initUI(Container frame) {
    	setLayout(new BorderLayout());

        paramPanel = new JPanel();
		add(paramPanel, BorderLayout.SOUTH);
        //paramPanel.setLayout(new GridLayout(1, 0, 8, 8));
        paramPanel.setLayout(new ProportionLayout());
        paramPanel.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));

        JPanel funcPanel = new JPanel();
        paramPanel.add(funcPanel);
        Border ttlFunc;
		ttlFunc = BorderFactory.createCompoundBorder(
        	BorderFactory.createTitledBorder("Function"),
            BorderFactory.createEmptyBorder(4, 10, 10, 10));
		funcPanel.setBorder(ttlFunc);
        //funcPanel.setLayout(new GridLayout(0, 2, 10, 10));
        funcPanel.setLayout(new FormLayout());

        JPanel codePanel = new JPanel();
        paramPanel.add(codePanel);
        //codePanel.setLayout(new GridLayout(0, 2, 5, 5));
        codePanel.setLayout(new FormLayout());
        Border ttlParams;
		ttlParams = BorderFactory.createCompoundBorder(
        	BorderFactory.createTitledBorder("Code"),
            BorderFactory.createEmptyBorder(4, 10, 10, 10));
		codePanel.setBorder(ttlParams);

        JPanel timingPanel = new JPanel();
        paramPanel.add(timingPanel);
        Border ttlTimings;
		ttlTimings = BorderFactory.createCompoundBorder(
        	BorderFactory.createTitledBorder("Timings"),
            BorderFactory.createEmptyBorder(4, 10, 10, 10));
		timingPanel.setBorder(ttlTimings);
        //timingPanel.setLayout(new GridLayout(0, 3, 5, 5));
        timingPanel.setLayout(new MyGridLayout(3, 8, 5));



        codePanel.add(new JLabel("Type"));
        String[] sigType = {"", "Pulse", "Space", "Shift"};
        codePanel.add(cbSigType = new JComboBox(sigType));
        cbSigType.setEnabled(false);

        codePanel.add(new JLabel("Frequency"));
        codePanel.add(tfFreq = new JTextField(6));
        tfFreq.setEditable(false);
        codePanel.add(new JLabel("Length"));
        codePanel.add(tfCodeLen = new JTextField(2));
        tfCodeLen.setEditable(false);
        codePanel.add(new JLabel("Code"));
        codePanel.add(tfCode = new JTextField(8));
        tfCode.setEditable(false);
        codePanel.add(new JLabel("Gap"));
        codePanel.add(tfGap = new JTextField(6));
        tfGap.setEditable(false);

        timingPanel.add(new JLabel(""));
        timingPanel.add(new JLabel("Pulse"));
        timingPanel.add(new JLabel("Space"));

        timingPanel.add(new JLabel("Header"));
        timingPanel.add(tfHeadP = new JTextField(4));
        tfHeadP.setEditable(false);
        timingPanel.add(tfHeadS = new JTextField(4));
        tfHeadS.setEditable(false);
        timingPanel.add(new JLabel("1"));
        timingPanel.add(tfOneP = new JTextField(4));
        tfOneP.setEditable(false);
        timingPanel.add(tfOneS = new JTextField(4));
        tfOneS.setEditable(false);
        timingPanel.add(new JLabel("0"));
        timingPanel.add(tfZeroP = new JTextField(4));
        tfZeroP.setEditable(false);
        timingPanel.add(tfZeroS = new JTextField(4));
        tfZeroS.setEditable(false);

        timingPanel.add(new JLabel("End"));
        timingPanel.add(tfEndP = new JTextField(4));
        tfEndP.setEditable(false);
        timingPanel.add(new JLabel(""));

        timingPanel.add(new JLabel("Stop"));
        timingPanel.add(tfStopP = new JTextField(4));
        tfStopP.setEditable(false);
        timingPanel.add(tfStopS = new JTextField(4));
        tfStopS.setEditable(false);

        funcPanel.add(new JLabel("Manuf."));
        String[] manList = {""};
        funcPanel.add(cbManuf = new JComboBox(manList));
        cbManuf.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
            	if (sneakyChange)
                	return;

				String mName = (String)cbManuf.getSelectedItem();
                if ((mName == null) || mName.equals("")) {
                	if (cbDev != null) {
                    	cbDev.removeAllItems();
                    }

                    if (cbFunc != null) {
                    	cbFunc.removeAllItems();
                    }
                }
                else {
	                cbDev.removeAllItems();
    	            cbDev.addItem("");
			        for (Enumeration it = irData.getDeviceList(mName);
                    	it.hasMoreElements(); ) {

                        IRDevice d = (IRDevice)(it.nextElement());
        				cbDev.addItem(d.getName());
		        	}

                    SignalParams signal = irData.getSignal(mName);
                    cbSigType.setSelectedItem(signal.type);
                    tfFreq.setText("" + signal.freq);
                    tfCodeLen.setText("" + signal.codeLen);
					tfCode.setText("");
                    tfHeadP.setText("" + signal.headP);
                    tfHeadS.setText("" + signal.headS);
                    tfOneP.setText("" + signal.oneP);
                    tfOneS.setText("" + signal.oneS);
                    tfZeroP.setText("" + signal.zeroP);
                    tfZeroS.setText("" + signal.zeroS);
                    tfEndP.setText("" + signal.endP);
                    tfStopP.setText("" + signal.stopP);
                    tfStopS.setText("" + signal.stopS);
                    tfGap.setText("" + signal.gap);
                }
			}
        });

        funcPanel.add(new JLabel("Device"));
        String[] devList = {""};
        funcPanel.add(cbDev = new JComboBox(devList));
        cbDev.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
            	if (sneakyChange)
                	return;

				String dName = (String)cbDev.getSelectedItem();
				String mName = (String)cbManuf.getSelectedItem();

                if ((dName == null) || dName.equals("")) {
                    if (cbFunc != null) {
                    	cbFunc.removeAllItems();
                    }
                }
                else {
	                cbFunc.removeAllItems();
    	            cbFunc.addItem("");
			        for (Enumeration it = irData.getFuncList(mName, dName);
                    	it.hasMoreElements(); ) {

						IRFunction f = (IRFunction)(it.nextElement());
        				cbFunc.addItem(f.name);
		        	}
                }
			}
        });

        funcPanel.add(new JLabel("Function"));
        String[] funcList = {""};
        funcPanel.add(cbFunc = new JComboBox(funcList));
        cbFunc.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
            	if (sneakyChange)
                	return;

				String fName = (String)cbFunc.getSelectedItem();
				String dName = (String)cbDev.getSelectedItem();

				String mName = (String)cbManuf.getSelectedItem();

                if ((fName == null) || fName.equals("")) {
                }
                else {
					IRFunction f = irData.getFunc(mName, dName, fName);
					tfCode.setText(Long.toHexString(f.code));

					SignalAnalysis analysis = getAnalyser(mName);

					if (analysis != null) {
						analysis.generateSignal(f.code, irData.getSignal(mName));
						if (analysis.isGenerated()) {
                        	bitstream = analysis.getIRBitstream();
							boolean done = analysis.analyzeSignal(
								irData.getSignal(mName), bitstream);

							tracePanel.repaint();

							System.out.println("Analysis="+done+", status="+analysis.getError());
						}
					}
                }
			}
        });

        funcPanel.add(new JLabel(""));

        timingPanel.layout();
        codePanel.layout();
        funcPanel.layout();

        paramPanel.layout();

        tracePanel = new MyCanvas();
		add(tracePanel, BorderLayout.CENTER);
        tracePanel.setBackground(Color.black);
    }

    public String getManuf() {
    	return (String)(cbManuf.getSelectedItem());
    }

    public IRData getIRData() {
    	return irData;
    }

    public void setDevice(String devName) {
    	cbDev.setSelectedItem(devName);
    }

    public void setFunction(String funcName, boolean refresh) {
    	if (funcName == null)
        	funcName = "";

    	sneakyChange = !refresh;
    	cbFunc.setSelectedItem(funcName);
        sneakyChange = false;
    }

	public void setCode(long code) {
		tfCode.setText(Long.toHexString(code));
    }

    public void setBitstream(IRBitstream bits) {
    	bitstream = bits;
        tracePanel.repaint();
    }

	public SignalAnalysis getAnalyser(String mName) {
		String module = irData.getModule(mName);

		if (module == null)
			return null;

		try {
			Class c = (Class)(manufModule.get(module));
			Object o = c.newInstance();
			return (SignalAnalysis)(c.newInstance());
		}
		catch (Exception ex) {
			System.out.println("Error:" + ex);
		}
		return null;
	}

	private void refreshDisplay() {
    	String mName = (String)cbManuf.getSelectedItem();
    	String dName = (String)cbDev.getSelectedItem();
    	String fName = (String)cbFunc.getSelectedItem();

		cbManuf.removeAllItems();
		cbManuf.addItem("");
		for (Enumeration it = irData.getManufList(); it.hasMoreElements(); ) {
			Manuf m = (Manuf)(it.nextElement());
			cbManuf.addItem(m.getName());
			String module = irData.getModule(m.getName());
			if (module != null) {
				try {
					Class c = Class.forName(module);
					//System.out.println("class is " + c.getName());
					manufModule.put(module, Class.forName(module));
				}
				catch (Exception ex) {
					System.out.println("Error: " + ex);
				}
			}
		}

		cbDev.removeAllItems();
		cbFunc.removeAllItems();

		tfFreq.setText("");
		tfCodeLen.setText("");
		tfCode.setText("");
		tfHeadP.setText("");
		tfHeadS.setText("");
		tfOneP.setText("");
		tfOneS.setText("");
		tfZeroP.setText("");
		tfZeroS.setText("");
		tfEndP.setText("");
		tfStopP.setText("");
		tfStopS.setText("");
		tfGap.setText("");

		cbManuf.setSelectedItem(mName);
        cbDev.setSelectedItem(dName);
        cbFunc.setSelectedItem(fName);
	}

    public void refreshManuf() {
    	String mName = (String)cbManuf.getSelectedItem();
    	cbManuf.setSelectedItem("");
    	cbManuf.setSelectedItem(mName);
    }

    public void refreshDevice() {
    	String dName = (String)cbDev.getSelectedItem();
    	cbDev.setSelectedItem("");
    	cbDev.setSelectedItem(dName);
    }

    public String getDevice() {
		return (String)cbDev.getSelectedItem();
    }

    public String getFunction() {
		return (String)cbFunc.getSelectedItem();
    }
}

