package pb.ir.ui;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import pb.comm.CommPort;
import pb.comm.LineReceiver;
import pb.comm.SerialConnectionException;

import pb.ir.irdata.IRBitstream;
import pb.ir.irdata.IRData;
import pb.ir.irdata.IRDevice;
import pb.ir.irdata.IRFunction;
import pb.ir.irdata.Manuf;
import pb.ir.irdata.SignalParams;
import pb.ir.analysis.SignalAnalysis;
import pb.ir.analysis.SignalAnalysisBasic;


/*
ToDo
  Error dialog on no manuf or analysis error
  Ask for save on exit after change
  Select current device by default in dialog and focus on function
  Add parameter to module. For Thomson, it is the prefix
  Bit timing needs tweaking
  Improve layout of dialog
  Write Phase analyser
*/

public class IRUI extends JPanel {
	JToolBar cmdPanel;
    JButton learnBtn, sendBtn;
	JFileChooser fc = new JFileChooser();
    DialogDevFunc dDevFunc = new DialogDevFunc();
    File currentFile = null;

    CommPort port;
    PortHandler pHandler = new PortHandler();

    IRMainUI mainUI;

    static final double picScale = 9.2;
    static final double zeroComp = 15;
    static final double oneComp = -15;

	class MainLayout implements LayoutManager {
    	Component p1;
    	JPanel p2, p3;

		public MainLayout(Component p1, JPanel p2, JPanel p3) {
        	this.p1 = p1;
        	this.p2 = p2;
        	this.p3 = p3;
        }

	    public void addLayoutComponent(String name, Component comp) {}

	    public void removeLayoutComponent(Component comp) {}

	    public Dimension preferredLayoutSize(Container parent) {
			return new Dimension(parent.getBounds().width,
				parent.getBounds().height);
	    }

	    public Dimension minimumLayoutSize(Container parent) {
			return new Dimension(parent.getBounds().width,
				parent.getBounds().height);
	    }

	    public void layoutContainer(Container parent) {
           	Rectangle parentSize = parent.getBounds();

            Dimension d1 = p1.getPreferredSize();
            Dimension d3 = p3.getPreferredSize();
            int y = 0, h = parentSize.height;;

            p1.setBounds(0, y, parentSize.width, d1.height);
            y += d1.height;
            h -= d1.height;
            p3.setBounds(0, parentSize.height - d3.height,
            	parentSize.width, d3.height);
            h -= d3.height;
            p2.setBounds(0, y, parentSize.width, h);
	    }
	}

    class PortHandler implements LineReceiver {
        int lastSig = 0;
        int prevSig = 0x80;
        IRBitstream bits;


        public void initHandler() {
            lastSig = 0;
            bits = new IRBitstream();
        }

		public void lineReceive(String line) {
    	    boolean halfByte = false;
        	char[] b = new char[1];
	        byte pix = 0;
    	   	int i = 0;

	        for (int inx = 0; inx < line.length(); inx++) {
                int sig = (int)(line.charAt(i));
                if (line.charAt(inx) == 0x80) {
                	// End of signal reached
					if (analyse(bits)) {
                    	mainUI.setBitstream(bits);
                    }
                    else {
                    	mainUI.setBitstream(bits);
                    }
                }
                else {
	                if ((sig & 0x80) == (prevSig & 0x80)) {
    	                lastSig += sig & 0x7f;
        	        }
            	    else {
                	    if (inx != 0 || lastSig != 0) {
                            if ((prevSig & 0x80) == 0) {
                            	lastSig += zeroComp;
                            }
                            else {
                            	lastSig += oneComp;
                            }

                            bits.addBitstream((int)(Math.round(lastSig * picScale)));
	                    }
    	                lastSig = sig & 0x7f;
        	        }
                }

                prevSig = sig;
	        }
    	}
    }

    public IRUI(JFrame frame) {
        port = new CommPort();
        try {
            port.openConnection();
            System.out.println("port opened");
            port.registerReceiver(pHandler);
        }
        catch (SerialConnectionException e) {
            System.out.println("Error:" + e);
        }

        setLayout(new BorderLayout());

        Action learnAction = new AbstractAction("Learn",
			new ImageIcon("learn.gif")) {

			public void actionPerformed(ActionEvent e) {
       	        pHandler.initHandler();
   	       	    port.sendString("R");
            }
	    };

        JMenuBar menu = new JMenuBar();
        frame.setJMenuBar(menu);

		JMenuItem fileMenu = new JMenu();
		fileMenu.setText("File");
		fileMenu.setActionCommand("File");
		menu.add(fileMenu);

		JMenuItem fileNewItem = new JMenuItem("New");
		fileMenu.add(fileNewItem);
		fileNewItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
            	mainUI.clear();
			}
		});

		JMenuItem fileOpenItem = new JMenuItem("Open...");
		fileMenu.add(fileOpenItem);
		fileOpenItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int returnVal = fc.showOpenDialog(IRUI.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currentFile = fc.getSelectedFile();

					try {
						FileInputStream is = new FileInputStream(currentFile);
                        mainUI.load(is);
					}
					catch (Exception ex) {
						JOptionPane.showMessageDialog(IRUI.this,
						    "Error:" + ex,
						    "File Open Error",
							JOptionPane.ERROR_MESSAGE);
                        ex.printStackTrace();
					}
				}
			}
		});

		JMenuItem fileSaveItem = new JMenuItem("Save");
		fileMenu.add(fileSaveItem);
		fileSaveItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					FileOutputStream is = new FileOutputStream(currentFile);
                    mainUI.getIRData().save(is);
                    is.close();
				}
				catch (Exception ex) {
					JOptionPane.showMessageDialog(IRUI.this,
					    "Error:" + ex,
					    "File Write Error",
						JOptionPane.ERROR_MESSAGE);
                    ex.printStackTrace();
				}
			}
		});

		JMenuItem fileSaveAsItem = new JMenuItem("Save As...");
		fileMenu.add(fileSaveAsItem);
		fileSaveAsItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int returnVal = fc.showOpenDialog(IRUI.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currentFile = fc.getSelectedFile();

					try {
						FileOutputStream is = new FileOutputStream(currentFile);
                        mainUI.getIRData().save(is);
	                    is.close();
					}
					catch (Exception ex) {
						JOptionPane.showMessageDialog(IRUI.this,
						    "Error:" + ex,
						    "File Write Error",
							JOptionPane.ERROR_MESSAGE);
                        ex.printStackTrace();
					}
				}
			}
		});

		JMenuItem fileExitItem = new JMenuItem("Exit");
		fileMenu.add(fileExitItem);
		fileExitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeWindow(e);
            }
        });

		JMenu commsMenu = new JMenu();
		commsMenu.setText("Comms");
		commsMenu.setActionCommand("Comms");
		menu.add(commsMenu);

        JMenuItem commsLearnItem = commsMenu.add(learnAction);
        commsLearnItem.setIcon(null);

		JMenuItem commsSendItem = new JMenuItem("Send");
		commsMenu.add(commsSendItem);

		JMenuItem commsSettingsItem = new JMenuItem("Settings...");
		commsMenu.add(commsSettingsItem);

        cmdPanel = new JToolBar();
		add(cmdPanel, BorderLayout.NORTH);
        cmdPanel.setFloatable(false);

        JButton learnBtn = cmdPanel.add(learnAction);
        learnBtn.setText("");
        learnBtn.setToolTipText("Read new IR signal from PIC");
        cmdPanel.add(new JButton(new ImageIcon("send.gif")));


        mainUI = new IRMainUI(frame);

		add(mainUI, BorderLayout.CENTER);

        fc.setFileFilter(new FileFilter() {
        	public boolean accept(File f) {
            	if (f.isDirectory()) {
                	return true;
                }

            	return f.getName().endsWith(".xml");
            }

            public String getDescription() {
            	return "XML Files";
            }
        });
    }

	private void closeWindow(ActionEvent event) {
		setVisible(false); // hide the Frame
		//dispose();	     // free the system resources
		System.exit(0);    // close the application
	}

	public static void main(String[] args) {
    	JFrame frame = new JFrame();
		frame.setSize(800,600);
		frame.setTitle("IRalyze v1.0");
		frame.setSize(800,600);
        frame.setIconImage(frame.getToolkit().createImage("pcir.gif"));

		IRUI irUI = new IRUI(frame);

		frame.getContentPane().add(irUI, BorderLayout.CENTER);
		frame.addWindowListener(new WindowAdapter() {
        	public void windowClosing(WindowEvent e) {
            	System.exit(0);
            }
        });

        frame.show();
	}

	public boolean analyse(IRBitstream bits) {
		String mName = mainUI.getManuf();
        IRData irData = mainUI.getIRData();

		if (mName != null && mName.length() > 0) {
			// Manuf selected
			Manuf m = irData.getManuf(mName);
			SignalAnalysis analysis = mainUI.getAnalyser(mName);
            if (analysis == null) {
            	analysis = new SignalAnalysisBasic();
            }

			analysis.analyzeSignal(irData.getSignal(mName), bits);

			if (analysis.isValid()) {
				long code = analysis.getCode();
                mainUI.setCode(code);

				IRDevice dev = m.getDevice(code);
                if (dev == null) {
                	// get new details
                    dDevFunc.show(m);
                    if (dDevFunc.isValid()) {
                    	String dName = dDevFunc.getDevice();
                        String fName = dDevFunc.getFunction();

	                    System.out.println("dev="+dDevFunc.getDevice()+", func="+dDevFunc.getFunction());
                        if ((dev = m.getDevice(dName)) == null) {
                        	dev = new IRDevice(dName);
                        	m.addDevice(dev);
							mainUI.refreshManuf();
                            mainUI.setDevice(dName);
                        }
                        mainUI.setDevice(dev.getName());
						IRFunction func;
                        if ((func = dev.getFunction(code)) == null) {
                        	func = new IRFunction();
                            func.name = fName;
                            dev.addFunction(func);
                            mainUI.refreshDevice();
                            mainUI.setFunction(fName, false);
                        }
                        func.code = code;
                        mainUI.setCode(code);
                    }
                }
                else {
                	// Known device. Set it and function
					mainUI.setDevice(dev.getName());
                    IRFunction func = dev.getFunction(code);
                    mainUI.setFunction(func.name, false);
                }
				return true;
			}

			if (!analysis.isValid()) {
System.out.println("Got error " + analysis.getError());
            	analysis = new SignalAnalysisBasic();
				analysis.analyzeSignal(new SignalParams(), bits);
            }
		}
		else {
System.out.println("No manuf");
		}
        /*
		if no manuf selected {
			for all manuf {
				if valid
					add to list
			}

			if no items in list {
				popup error
			}
			else if multiple items {
				let user select one
			}

			if manuf set {
				getNewDetails();
			}
		}
        */
        return false;
	}
}

